/*
 * $Id: executor.cc,v 1.3 2008/10/22 10:12:41 hschaa Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	SCPM executor
 * File				 :	scpm.cc
 * Description :	executes external binaries
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#include "executor.hh"
#include "scpm_conf.hh"
#include "log.hh"
#include <sstream>
#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <vector>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#include <sys/stat.h>

using namespace std;

#define module_name "executor"

Executor::Executor( string command )
{
	string cmd;
	fname = command.substr( 0, (command.find(" ") == string::npos ? command.size() : command.find(" ") ) );
	if ( command.find(" ") == string::npos ) 
		cmd="";
	else
		cmd=command.substr( command.find(" ")+1 );

		// now create arg array
		// first arg is always the binary name
	args[0] = strdup( (fname.substr( fname.find_last_of("/")+1 )).c_str() );

	for( num_args=1; cmd.find(" ") != string::npos; num_args++ ) {
		args[num_args] = strdup( (cmd.substr( 0, cmd.find(" ") )).c_str() );
		cmd = cmd.substr( cmd.find(" ")+1 );
	}

		// last argument may left
	if ( cmd.length()>0 ) {
		args[num_args++] = strdup( cmd.c_str() );
	}

		// terminate array
	args[num_args] = new char( sizeof(NULL) );
	args[num_args] = NULL;

		//cout << "Executor: " << fname << endl;
		//int i=0;
		//while ( args[i] != NULL ) cout << "args[" << i << "]: " << args[i++] << endl;
}

Executor::~Executor( )
{
	for ( int i=0; i<=num_args; i++ )
		delete args[i];
}
/*
	int Executor::Run( streambuf *outbuf, streambuf *errbuf ) throw( exec_failed )
	{
	pid_t pid;

	pid = fork();

	if ( pid == -1 ) throw exec_failed();

	if ( pid == 0 ) {
	cout.rdbuf( outbuf );
	cerr.rdbuf( errbuf );
	string binary = fname.substr( fname.find_last_of("/")+1 );
	execl( fname.c_str(), binary.c_str(), NULL );
	abort();
	}

	int status;
	waitpid( pid, &status, 0 );

	if ( !WIFEXITED(status) ) {
		// child could not execl
		throw exec_failed();
		}

		return WEXITSTATUS(status);
		}
*/
/*
	int Executor::Run( vector<string> &out_strings, iostream &command_errstream ) throw( exec_failed )
	{
	return 0;
	}
*/

int Executor::Run( vector<string> &out_strings, vector<string> &err_strings ) throw( exec_failed )
{
	pid_t pid;
	char outfile[30] = "/tmp/scpm_script_out.XXXXXX";
	char errfile[30] = "/tmp/scpm_script_err.XXXXXX";
	if ( mktemp( outfile ) == NULL ) throw exec_failed();
	if ( mktemp( errfile ) == NULL ) {
		unlink( outfile );
		throw exec_failed();
	}

	int fdout = open( outfile, O_CREAT|O_RDWR, 0600 );
	if ( fdout == -1 ) {
		unlink( outfile );
		unlink( errfile );
		throw exec_failed();
	}
	int fderr = open( errfile, O_CREAT|O_RDWR, 0600 );
	if ( fderr == -1 ) {
		close( fdout );
		unlink( outfile );
		unlink( errfile );
		throw exec_failed();
	}

	pid = fork();

	if ( pid == -1 ) throw exec_failed();

	if ( pid == 0 ) {
			// this may be needed for running scpm via suid
		setuid( 0 );
		dup2( fdout, 1 );
		dup2( fderr, 2 );
		execvp( fname.c_str(), args );
		cerr << "execvp() failed" << endl;
		cerr << strerror( errno ) << endl;
		abort();
	}

	int status=0;
	waitpid( pid, &status, 0 );

	if ( !WIFEXITED(status) ) {
			// child could not execl
		close( fdout );
		close( fderr );
		unlink( outfile );
		unlink( errfile );
		throw exec_failed();
	}

	lseek( fdout, 0, SEEK_SET );
	lseek( fderr, 0, SEEK_SET );

	char buf[256];
	int i=0;

	while ( read( fdout, &buf[i], 1 ) > 0 ) {
		if ( buf[i] == '\n' ) {
			buf[i]='\0';
			out_strings.push_back( buf );
			i=0;
		}
		else if (i>254) {
			buf[i]='\0';
			out_strings.push_back( buf );
			i=0;
			break;
		}
		else i++;
	}
	while ( read( fderr, &buf[i], 1 ) > 0 ) {
		if ( buf[i] == '\n' ) {
			buf[i]='\0';
			err_strings.push_back( buf );
			i=0;
		}
		else if (i>254) {
			buf[i]='\0';
			err_strings.push_back( buf );
			i=0;
			break;
		}
		else i++;
	}

	close( fdout );
	close( fderr );
	unlink( outfile );
	unlink( errfile );
	return WEXITSTATUS(status);
}

int Executor::Run( string &logdir, bool verbose ) throw( exec_failed )
{
	pid_t pid;
	char outfile[30] = "/tmp/scpm_script_out.XXXXXX";
	if ( mktemp( outfile ) == NULL ) throw exec_failed();

	int fdout = open( outfile, O_CREAT|O_RDWR, 0600 );
	if ( fdout == -1 ) {
		LOGDEBUG( string("could not open ")+outfile+" for writing" );
		LOGDEBUG( strerror( errno ) );
		unlink( outfile );
		throw exec_failed();
	}

	pid = fork();
	if ( pid == -1 ) {
		close( fdout );
		unlink( outfile );
		throw exec_failed();
	}

	if ( pid == 0 ) {
			// this may be needed for running scpm via suid
		setuid( 0 );
		if ( !verbose ) close( 1 );
		dup2( fdout, 2 );
		execvp( fname.c_str(), args );
		cerr << "execvp() failed" << endl;
		cerr << strerror( errno ) << endl;
		abort();
	}

	int status=0;
	waitpid( pid, &status, 0 );

	if ( !WIFEXITED(status) ) {
			// child could not exec
		close( fdout );
		unlink( outfile );
		throw exec_failed();
	}
	lseek( fdout, 0, SEEK_SET );

	char buf[256];
	int i=0;

	while ( read( fdout, &buf[i], 1 ) > 0 ) {
		if ( buf[i] == '\n' ) {
			buf[i]='\0';
			logdir=buf;
			break;
		}
		else if ( i>254 ) {
				// huh? this should not happen
			logdir.clear();
			break;
		}
		else i++;
	}

	close( fdout );
	unlink( outfile );
	return WEXITSTATUS(status);
}


Script::Script( const string &filename ) : scriptfile( filename )
{
}

void Script::Exec( )
{
	string logdir;

	LOGDEBUG( "running script "+scriptfile );
	try {
		Executor e( SCPM_conf::GetHandle()->GetLibDir()+"/run_script "+scriptfile );
		if ( e.Run( logdir, SCPM_conf::GetHandle()->Verbose() ) != 0 ) {
			LOGWARN( "bad exit status of run_script "+scriptfile );
		}
	}
	catch( Executor::exec_failed ) {
		LOGWARN( "could not execute script "+scriptfile );
		return;
	}

		// analyse script logs
	if ( logdir.size() == 0 ) {
		LOGWARN( "could not locate script logs, this should not happen" );
		return;
	}
	struct stat statbuf;
	if ( ::stat( (logdir+"/messages").c_str(), &statbuf ) == 0 ) {
			// we have to log messages
		ifstream logs( (logdir+"/messages").c_str() );
		string log_class;
		Log *log = Log::GetHandle();
		while ( logs >> log_class ) {
			Log::logclass_t lc;
			if ( log_class == "debug" ) lc = Log::CLASS_DEBUG;
			else if ( log_class == "info" ) lc = Log::CLASS_INFO;
			else if ( log_class == "warning" ) lc = Log::CLASS_WARNING;
			else if ( log_class == "ERROR" ) lc = Log::CLASS_ERROR;
			char buf[200];
			logs >> ws;
			logs.getline( buf, 199 );
			log->WriteRawMessage( lc, buf );
		}
		logs.close();
	}

	if ( ::stat( (logdir+"/exitcode").c_str(), &statbuf ) == 0 ) {
		ifstream exits( (logdir+"/exitcode").c_str() );
		string buf;
		exits >>	buf;
		if ( buf != "0" ) LOGWARN( "script exited with code "+buf );
		exits.close();
	}

	if ( ::stat( (logdir+"/output").c_str(), &statbuf ) == 0 ) {
		char buf[255];
		ifstream in( (logdir+"/output").c_str() );
		if (in.good()) {
			LOGDEBUG( "output of "+scriptfile+" follows" );
			while (in.getline( buf, 254 ))
				LOGDEBUG( buf );
		}
		in.close();
	}

	bool stop_op=false;
	if ( ::stat( (logdir+"/aborted").c_str(), &statbuf ) ==0 ) {
			// uh, the script says STOP
		LOGWARN( scriptfile+" script aborted SCPM operation" );
		LOGWARN( "this may result in an inconsistent system (read documenation)" );
		stop_op = true;
	}

	DIR *dp;
	struct dirent *ep;
	dp = ::opendir( logdir.c_str() );
	if ( dp ) {
		while ( (ep = readdir(dp)) ) 
			::unlink( (logdir+"/"+ep->d_name).c_str() );
		::closedir( dp );
	}
	::rmdir( logdir.c_str() );
	
	if (stop_op) LOGERR( "SCPM aborted by &"+scriptfile );
}
