/*
 * $Id: journal.hh,v 1.3 2006/09/26 11:44:42 hschaa Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	Journal
 * File				 :	journal.hh
 * Description :	provides locking and operation logging
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef JOURNAL_H
#define JOURNAL_H

#include <string>
#include <vector>

#define jrnl Journal::GetHandle()

class Journal
{
public:

	friend class SCPM;

	enum status_t {
		STATUS_CLEAN,
		STATUS_LOCKED,
		STATUS_STALE_LOCK,
		STATUS_RECOVER_NEEDED
	} ;

	struct entry {
		int command;
		std::vector<std::string> args;
		bool replay;
	};

		// journalized commands
	static const int cmd_set_global_key					 = 1;
	static const int cmd_save_install_info			 = 2;
	static const int cmd_write_boot_info				 = 3;
	static const int cmd_clear_boot_info				 = 4;
	static const int cmd_clear_all							 = 5;
	static const int cmd_reload_db							 = 6;
	static const int cmd_prepare_system					 = 7;
	static const int cmd_message								 = 8;
	static const int cmd_resource_save					 = 10;
	static const int cmd_resource_patch					 = 11;
	static const int cmd_resource_restore				 = 12;
	static const int cmd_resource_stop					 = 13;
	static const int cmd_resource_start					 = 14;
	static const int cmd_resource_add						 = 15;
	static const int cmd_resource_add_type			 = 16;
	static const int cmd_resource_delete				 = 17;
	static const int cmd_resource_add_dep				 = 18;
	static const int cmd_resource_clean_deps		 = 19;
	static const int cmd_resource_add_data			 = 20;
	static const int cmd_resource_clear_data		 = 21;
	static const int cmd_resource_create				 = 22;
	static const int cmd_resource_drop					 = 23;
	static const int cmd_resource_restore_backup = 24;
	static const int cmd_resource_add_backup     = 25;
	static const int cmd_resource_remove_backup  = 26;
	static const int cmd_profile_add						 = 30;
	static const int cmd_profile_delete					 = 31;
	static const int cmd_profile_add_resource		 = 32;
	static const int cmd_profile_delete_resource = 33;
	static const int cmd_profile_clear_resources = 34;
	static const int cmd_profile_set_key				 = 35;
	static const int cmd_profile_set_script			 = 36;
	static const int cmd_file_save							 = 40;
	static const int cmd_file_restore						 = 41;
	static const int cmd_file_copy							 = 42;
	static const int cmd_file_delete						 = 43;
	static const int cmd_call										 = 50;
	static const int cmd_script									 = 51;
	static const int cmd_group_activate					 = 60;
	static const int cmd_group_deactivate				 = 61;
	static const int cmd_group_set							 = 62;
	static const int cmd_group_delete						 = 63;
	static const int cmd_group_rename						 = 64;
	static const int cmd_group_reset						 = 65;
	static const int cmd_group_reset_all				 = 65;
	static const int cmd_group_set_active				 = 66;

		// some exceptions
	struct no_more_entries{};
	struct load_failed
	{
	public:
		load_failed( std::string r ) { reason=r; };
		std::string reason; 
	};

	static Journal *GetHandle() { return journal_handle; }

	status_t GetStatus() { return status; }
	bool MakeLock( );
	void Unlock( );
	void New( );
	void Ready( std::string magic );
	void AddEntry( int command, std::string arg1="", std::string arg2="",
								 std::string arg3="", std::string arg4="", std::string arg5="" );
	entry &NextEntry( bool mark_as_done=true ) throw( no_more_entries );
	entry &PrevEntry( bool mark_as_undone=true ) throw( no_more_entries );
	void Load( std::string magic ) throw( load_failed );
	void Delete( );
	void Done( unsigned int mark );
	int Steps() { return entry_stack.size()-stack_ptr; };
	bool IsReady( ) { return ready; };

private:
		
	Journal();
	~Journal();
	
	static const char *hdr;
	static const char *sep;

	static Journal *journal_handle;
	status_t status;
	std::vector<entry> entry_stack;
	unsigned int stack_ptr;
	unsigned int done;
	bool lock_created;
	bool ready;
};

#endif
