/*
 * $Id: log.hh,v 1.1.1.1 2005/05/11 09:03:12 gleissner Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	Log
 * File				 :	log.hh
 * Description :	log class specification
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef LOG_H
#define LOG_H

#include <string>
#include <sstream>
#include <scpm_types.h>

#define LOGMSG(loglev,msg) Log::GetHandle()->WriteMessage(module_name,loglev,msg)
#define LOGDEBUG(msg) Log::GetHandle()->WriteMessage(module_name,Log::CLASS_DEBUG,msg)
#define LOGINFO(msg) Log::GetHandle()->WriteMessage(module_name,Log::CLASS_INFO,msg)
#define LOGWARN(msg) Log::GetHandle()->WriteMessage(module_name,Log::CLASS_WARNING,msg)
#define LOGERR(msg) Log::GetHandle()->WriteMessage(module_name,Log::CLASS_ERROR,msg)
#define LOGTRANS(msg) Log::GetHandle()->trans(msg)

class Log
{
public:

	friend class SCPM;

	typedef int logtype_t;
	typedef int logclass_t;
	typedef int logmode_t;
	typedef int exitmode_t;

	struct scpm_panic{};

	static Log * GetHandle( ) { return log_handle; };

	static const int TYPE_QUIET	 = 10;
	static const int TYPE_NORMAL = 20;
	static const int TYPE_VERBOSE= 30;
	static const int TYPE_DEBUG	 = 40;

	static const int CLASS_ERROR	 = 10;
	static const int CLASS_WARNING = 20;
	static const int CLASS_INFO		 = 30;
	static const int CLASS_DEBUG	 = 40;

	static const int MODE_NONE		= 0x00;
	static const int MODE_STDERR	= 0x01;
	static const int MODE_SYSLOG	= 0x02;
	static const int MODE_FILE		= 0x04;

	static const int EXIT_ERROR		= 0;
	static const int EXIT_WARNING = 1;

	void SetLog( logtype_t log_type, logmode_t logmode );
	void WriteMessage( std::string module_name, logclass_t log_class, std::string message,
										 std::string action="" );
	void WriteRawMessage( logclass_t log_class, std::string message, std::string action="" );
	std::string GetLastMessage();
	bool Aborted( ) { return aborted; };
	void ClearAborted( ) { aborted = false; };

	static const struct endlog_t{} endlog;
	static const struct debug_t{} debug;
	static const struct info_t{} info;
	static const struct warning_t{} warning;
	static const struct error_t{} error;

		// logging operations
	Log &operator<<( std::string s);
	Log &operator<<( char c );
	Log &operator<<( long i );
	Log &operator<<( unsigned long i );
	Log &operator<<( int i) {
		return this->operator<<(static_cast<signed long>(i)); }
	Log &operator<<( unsigned int i) { 
		return this->operator<<(static_cast<unsigned long>(i)); }

		// manipulators (kind of ...)
	Log &operator<<( endlog_t &a );
	Log &operator<<( debug_t &a );
	Log &operator<<( info_t &a );
	Log &operator<<( warning_t &a );
	Log &operator<<( error_t &a );
	
	std::string trans( std::string raw );
	std::string untrans( std::string raw );

private:
		
	Log();
	~Log();
	
	static Log *log_handle;

	static std::string last_error;
	static int log_type;
	static int log_mode;
	static int exit_mode;
	static int current_class;

	static std::stringstream buf;

	bool aborted;
};

#endif
