/*
 * $Id: modlib.cc,v 1.2 2006/09/21 07:47:58 hschaa Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	Module library
 * File				 :	modlib.c
 * Description :	support functions for resource type modules
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#include "modlib.hh"
#include "executor.hh"
#include "log.hh"
#include "scpm_conf.hh"
#include <cstdlib>
#include <cstdio>
#include <string>
#include <fcntl.h>
#include <iostream>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#define CRYPTOPP_ENABLE_NAMESPACE_WEAK 1
#include <cryptopp/md5.h>
#include <cryptopp/files.h>
#include <cryptopp/filters.h>
#include <cryptopp/hex.h>

using namespace std;
using namespace CryptoPP::Weak1;
using namespace CryptoPP;

#define module_name "modlib"

string Modlib::GetMD5sum( string filename )
{

    MD5 hash;
    byte buffer[2 * MD5::DIGESTSIZE];

    FileSource f(filename.c_str(), true,
             new HashFilter(hash,
             new HexEncoder(new ArraySink(buffer,2 * MD5::DIGESTSIZE), false)));

    return string((const char*)buffer,2 * MD5::DIGESTSIZE);
}

string Modlib::GetMD5sum( istream &input )
{
    MD5 hash;
    byte buffer[2 * MD5::DIGESTSIZE];
    FileSource f(input, true, new HashFilter(hash, new HexEncoder(new ArraySink(buffer,2 * MD5::DIGESTSIZE), false)));
    return string((const char*)buffer,2 * MD5::DIGESTSIZE);

}

int Modlib::CallHandler( string resource_type, string resource_name,
												 string action, string args, vector<string> &output, vector<string> &errout )
	throw( handler_not_available )
{
	string filename;
	struct stat buf;
	int retval;
	SCPM_conf *conf = SCPM_conf::GetHandle();

	filename = conf->GetLibDir()+"/resource_types/"+resource_type+"/"+action+"/"+resource_name;

	if ( stat( filename.c_str(), &buf ) == -1 ) {
		filename = conf->GetLibDir()+"/resource_types/"+resource_type+"/"+action+"/generic";
		if ( stat( filename.c_str(), &buf ) == -1 ) {
			throw handler_not_available();
		}
	}

	string tmp;
	if ( args.size() >0 )
		tmp = " "+args;

	Executor e( filename+" "+resource_name+tmp );

	try {
		retval = e.Run( output, errout );
	}
	catch( Executor::exec_failed ) {
		LOGDEBUG( "execution of "+filename+" failed" );
		if ( !output.empty() ) {
			LOGDEBUG( "output was" );
			for (unsigned int i=0; i<output.size(); i++)
				LOGDEBUG( output[i] );
		}
		throw handler_not_available();
	}
	return retval;
}

bool Modlib::HasHandler( string resource_type, string resource_name, string action )
{
	string filename;
	struct stat buf;
	SCPM_conf *conf = SCPM_conf::GetHandle();

	filename = conf->GetLibDir()+"/resource_types/"+resource_type+"/"+action+"/"+resource_name;

	if ( stat( filename.c_str(), &buf ) == -1 ) {
		filename = conf->GetLibDir()+"/resource_types/"+resource_type+"/"+action+"/generic";
		if ( stat( filename.c_str(), &buf ) == -1 ) {
			return false;
		}
	}
	return true;
}
