/*
 * $Id: profile.hh,v 1.1.1.1 2005/05/11 09:03:12 gleissner Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	Profile
 * File				 :	profile.hh
 * Description :	Profile class description
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef PROFILE_H
#define PROFILE_H

#include <scdb.hh>
#include "resource.hh"
#include "log.hh"
#include <scpm_types.h>
#include <string>

class Profile
{
public:

	Profile( std::string profile_name );
	~Profile( );

	std::string GetName( ) { return name; };

	bool IsClean( );
	bool IsActive( );
	void SetActive( );
	void SetInactive( );
	void Remove( );
	void Restore( );
	void SetRoot( std::string root, bool is_profile_root );
	std::vector<std::string> GetResources( std::string resource_type );
	std::vector<std::string> GetModifiedResources( std::string resource_type );
	void DetermineResourceActions( );

		// script helpers
	void ScriptRunPreStart ( );
	void ScriptRunPostStart( );
	void ScriptRunPreStop	 ( );
	void ScriptRunPostStop ( );
	void ScriptSetPreStart ( std::istream &input );
	void ScriptSetPostStart( std::istream &input );
	void ScriptSetPreStop	 ( std::istream &input );
	void ScriptSetPostStop ( std::istream &input );
	void ScriptGetPreStart ( std::ostream &output );
	void ScriptGetPostStart( std::ostream &output );
	void ScriptGetPreStop	 ( std::ostream &output );
	void ScriptGetPostStop ( std::ostream &output );
	void ScriptUnsetPreStart ( );
	void ScriptUnsetPostStart( );
	void ScriptUnsetPreStop	 ( );
	void ScriptUnsetPostStop ( );
	bool HasScript( std::string scriptname );

private:

	const std::string module_name;
	SCDB *db;
	Log *log;
	std::string name;
	bool clean;
	std::vector<std::string> resources;
	std::vector<Resource*> resources_to_start;
	std::vector<Resource*> resources_to_stop;
	std::vector<Resource*> resources_keep_running;

	void SaveScript( std::string scriptname, std::istream &input );
	void DumpScript( std::string scriptname, std::ostream &output );
	void RemoveScript( std::string scriptname );
	void RunScript( std::string scriptname );

};

#endif
