/*
 * $Id: progress.cc,v 1.1.1.1 2005/05/11 09:03:12 gleissner Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	SCPM main engine
 * File				 :	progress.cc
 * Description :	helper class for progress visualization
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002,2003 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#include "progress.hh"
#include "log.hh"
#include <string.h>

using namespace std;

Progress::Progress( ostream &info_out, ostream &hash_out )
		: output_info(info_out), output_hash(hash_out)
{
//	output_info.rdbuf( info_out.rdbuf() );
//	output_hash.rdbuf( hash_out.rdbuf() );
	abs_length	= 100;
	abs_actual	= 0;
	abs_printed = 0;
	vlev = 0;
	seg_length	= 0;
	seg_actual	= 0;
	seg_percent = 0;
	seg_printed = 0;
	new_bar = true;
}

Progress::~Progress( )
{
	output_info.flush();
}

void Progress::SetOutputMode( int flags )
{
	output_flags = flags;
	if ( flags & verbose ) vlev=1;
	if ( flags & quiet ) vlev=-1;
}

void Progress::Print( const string &info, int verbose )
{
	if ( verbose <= vlev ) {
		output_info << Log::GetHandle()->trans(info) << " ";
		output_info.flush();
	}
}

void Progress::Newline( int verbose )
{
	if ( verbose <= vlev ) output_info << endl;
}

void Progress::SetBar( int bar_length )
{
	abs_length	= bar_length;
	abs_actual	= 0;
	abs_printed = 0;
		// per default, we create a 100 percent segment
	seg_length = bar_length;
	seg_actual = 0;
	seg_percent = 100;
	seg_printed = 0;
	new_bar = true;
}

void Progress::SetSegment( int percent )
{
	if (!new_bar) while ( seg_actual < seg_length ) this->BarStep();
	new_bar = false;
	seg_percent = percent;
	seg_actual	= 0;
	seg_printed = 0;
}

void Progress::SetSegmentSteps( int steps )
{
	if ( steps == 0 )
		seg_length = 1;
	else
		seg_length = steps;
}

void Progress::ResetBar( )
{
	if ( !new_bar && abs_actual < abs_length ) {
		seg_actual = abs_actual;
		seg_length = abs_length;
		seg_printed = abs_printed;
		seg_percent = 100;
		while ( abs_actual < abs_length ) this->BarStep();
	}
		// creating a default bar
	this->SetBar( 100 );
}

void Progress::BarStep( )
{
	seg_actual++;
	abs_actual++;
	if (seg_actual-1 == seg_length) return;
	if (abs_printed == abs_length) return; 
	if ( ! (output_flags & hash_marks) ) return;
	if ( seg_actual/(float)seg_length*seg_percent > seg_printed ) {
		int missing=(int)((seg_actual/(float)seg_length)*seg_percent-seg_printed);
		for ( int j=0; j < missing; j++) {
			output_hash << "#";
			seg_printed++;
			abs_printed++;
		}
	}
	output_hash.flush();
	new_bar=false;
}
