/*
 * $Id: resource.cc,v 1.2 2006/09/21 07:47:58 hschaa Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	Resource handler
 * File				 :	resource.cc
 * Description :	implements resource highlevel action class container
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002-2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#include "resource.hh"
#include "scdb.hh"
#include "journal.hh"

using namespace std;

#define module_name "resource"

Resource::Resource( string name_string, string type_string, bool allow_missing ) throw( resource_unknown ) :
		name(name_string), type(type_string), missok(allow_missing)
{
	profile = "__unset";
	resource_root = "";

	SCDB *db = SCDB::GetHandle();
	vector<string> data_entries;
	vector<string> data_values;

	try {
		db->ResourceGetData( name, type, data_entries, data_values );
		for ( unsigned int i=0; i<data_entries.size() ; i++ ) {
			data_element d;
			d.data_name = data_entries[i];
			d.data_value = data_values[i];
			resource_data.push_back( d );
		}
	}
	catch( SCDB::db_action_failed &e ) {
		LOGDEBUG( "scdb error ("+e.reason+")" );
		LOGDEBUG( "collecting resource data failed, maybe resource does not exist" );
			// we do not abort object creation in case of a db error (when allow_missing is set)
			// reason: in journalized op the node is not in the db yet,
			// but we need the GetDependencies function before
		if (!missok) throw resource_unknown();
	}
}

Resource::~Resource( )
{
}


bool Resource::Active( ) throw( not_available )
{
	throw not_available();
}

bool Resource::Running( ) throw( not_available )
{
	throw not_available();
}

bool Resource::Modified( ) throw( not_available )
{
	throw not_available();
}

bool Resource::NeedsUpdate( ) throw( not_available )
{
	throw not_available();
}

bool Resource::UpToDate( ) throw( not_available )
{
	throw not_available();
}

bool Resource::Save( ) throw( not_available )
{
	throw not_available();
}

bool Resource::Restore( ) throw( not_available )
{
	throw not_available();
}

bool Resource::Patch( const string &ref_profile ) throw( not_available )
{
	throw not_available();
}

bool Resource::Start( ) throw( not_available )
{
	throw not_available();
}

bool Resource::Stop( ) throw( not_available )
{
	throw not_available();
}

bool Resource::Restart( ) throw( not_available )
{
	throw not_available();
}

void Resource::UpdateData( ) throw( not_available )
{
	throw not_available();
}

vector< pair<time_t, string> > Resource::GetBackupVersions( ) throw(not_available)
{
  throw not_available();
}

vector<string> Resource::GetDependencies( string resource_type ) throw( not_available )
{
	throw not_available();
}

int Resource::GetStartPriority( ) throw( not_available )
{
	throw not_available();
}

int Resource::GetStopPriority( ) throw( not_available )
{
	throw not_available();
}

void Resource::ShowChanges( ostream &output, bool backup, string version ) throw( not_available )
{
	throw not_available();
}

void Resource::WriteData( )
{
	SCDB *db = SCDB::GetHandle();
	try {
		db->ResourceClearData( name, type );

		if ( resource_data.size() == 0 ) {
				// no resource specific data available, so do nothing
			return;
		}

			// add data
		for (unsigned int i=0; i < resource_data.size(); i++) {
			db->ResourceAddData( name, type, resource_data[i].data_name, resource_data[i].data_value );
		}
	}
	catch( SCDB::db_action_failed &e ) {
		LOGDEBUG( "scdb error ("+e.reason+")" );
		LOGWARN( "could not write resource specific data" );
	}
}

void Resource::SetResourceRoot( string root )
{
	if ( root.at( root.length()-1 ) != '/' )
		resource_root = root + "/";
	else
		resource_root = root;
}

bool Resource::ModifiedInProfile( const string &dest_profile ) throw( Resource::not_available )
{
	throw not_available();
}

bool Resource::HasSystemAlternative( ) throw( not_available )
{
	throw not_available();
}

bool Resource::HasSystemBackup( ) throw( not_available )
{
	throw not_available();
}

bool Resource::AddBackup( ) throw( not_available )
{
	throw not_available();
}

bool Resource::HasBackup( ) throw( not_available )
{
	throw not_available();
}

bool Resource::RestoreBackup( const string * version ) throw( not_available )
{
	throw not_available();
}

bool Resource::RemoveBackup( const string * version ) throw( not_available )
{
	throw not_available();
}

bool Resource::Refresh( ) throw( not_available )
{
	throw not_available();
}

bool Resource::Export( const string &root ) throw( not_available )
{
	throw not_available();
}

bool Resource::Import( const string &root ) throw( not_available )
{
	throw not_available();
}
