/*
 * $Id: resource.hh,v 1.2 2006/09/21 07:47:58 hschaa Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	Resource handler
 * File				 :	resource.cc
 * Description :	defines resource class container
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002-2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef RESOURCE_H
#define RESOURCE_H

#include <scdb.hh>
#include <string>
#include <vector>

class Resource
{
	friend class Resource_helpers;

public:

	struct not_available {};
	struct resource_unknown {};

	virtual ~Resource( );

	std::string &GetName( ) { return name; };
	std::string &GetProfile( ) { return profile; };
	virtual std::string GetType( ) { return "__unknown"; };

		// Resource status functions
	virtual bool Modified( ) throw( not_available );
	virtual bool UpToDate( ) throw( not_available );
	virtual bool Active( )	throw( not_available );
	virtual bool Running( ) throw( not_available );
	virtual bool NeedsUpdate( ) throw( not_available );
	virtual bool ModifiedInProfile( const std::string &dest_profile ) throw( not_available );
	virtual bool HasSystemAlternative( ) throw( not_available );
	virtual bool HasSystemBackup( ) throw( not_available );
	virtual bool HasBackup( ) throw( not_available );

		// Resource action functions
	virtual bool Save( ) throw( not_available );
	virtual bool Restore( ) throw( not_available );
	virtual bool Start( ) throw( not_available );
	virtual bool Stop( ) throw( not_available );
	virtual bool Restart( ) throw( not_available );
	virtual bool Patch( const std::string &ref_profile ) throw( not_available );
	virtual bool AddBackup( ) throw( not_available );
	virtual bool RestoreBackup( const std::string * version = NULL ) throw( not_available );
	virtual bool RemoveBackup( const std::string * version = NULL ) throw( not_available );
	virtual bool Export( const std::string &root ) throw( not_available );
	virtual bool Import( const std::string &root ) throw( not_available );

	virtual std::vector< std::pair<time_t, std::string> > GetBackupVersions( ) throw (not_available);
	virtual std::vector<std::string> GetDependencies( std::string resource_type ) throw( not_available );
	virtual int	 GetStartPriority( ) throw( not_available );
	virtual int	 GetStopPriority( ) throw( not_available );

	virtual void ShowChanges( std::ostream &output, bool backup=false, std::string version="" ) throw( not_available );
	virtual bool Refresh( ) throw( not_available );

		// Resource database functions
	virtual void UpdateData( ) throw( not_available );
	void WriteData( );

		// Profile specific data
	void SetProfile( std::string profile_name ) { profile = profile_name; };
	void SetResourceRoot( std::string root );

protected:
		
	Resource( std::string name_string, std::string type_string, bool allow_missing ) throw( resource_unknown );

	std::string name;
	std::string type;
	bool missok;

	struct data_element {
		std::string data_name;
		std::string data_value;
	};

	std::vector<data_element> resource_data;

	std::string profile;
	std::string resource_root;

private:

};

#endif

