/*
 * $Id: resource_helpers.cc,v 1.1.1.1 2005/05/11 09:03:12 gleissner Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	SCPM main engine
 * File				 :	resource_helpers.cc
 * Description :	implements resource specific helper routines
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#include <resource_helpers.hh>
#include <string>
#include "executor.hh"
#include "log.hh"
#include "scpm_conf.hh"
#include <file_old.hh>
#include <service_old.hh>

using namespace std;

#define module_name "resource_helpers"

Resource_helpers::~Resource_helpers( )
{
}

vector<string> Resource_helpers::GetResourceNames( string resource_type )
{
	vector<string> out;
	vector<string> err;
	SCPM_conf *conf = SCPM_conf::GetHandle();
	string cmd_line = conf->GetLibDir() + "/resource_types/"+resource_type+"/get_all";
	
	Executor e( cmd_line );
	try {
		e.Run( out, err );
	}
	catch( Executor::exec_failed ) {
		LOGWARN( "could not determine resource names" );
		if ( err.size() ) LOGDEBUG( "output of get_all script:" );
		for (unsigned int i=0; i<err.size(); i++)
			LOGDEBUG( err[i] );
		LOGERR( "could not determine resource names for resource type &"+resource_type );
		vector<string> foo;
		foo.clear();
		return foo;
	}

	if ( out.size() == 0 ) {
		LOGWARN( "no resources for resource type "+resource_type+" available" );
		LOGDEBUG( "get_all script returned no resources" );
		if ( err.size() ) LOGDEBUG( "get_all error messages:" );
		for (unsigned int i=0; i<err.size(); i++)
			LOGDEBUG( err[i] );
	}

	return out;
}

Resource *Resource_helpers::CreateObject( string resource_type, 
																					string resource_name,
																					bool allow_missing )
	throw ( type_invalid, Resource::resource_unknown )
{
	if ( resource_type == "file" ) 
		return ::new File( resource_name, "file", allow_missing );
	else if ( resource_type == "service" )
		return ::new Service( resource_name, "service", allow_missing );
	else
		throw type_invalid();
}
