/*
 * $Id: scpm_conf.hh,v 1.2 2005/12/13 16:30:15 gleissner Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	SCPM main engine
 * File				 :	scpm_conf.hh
 * Description :	Interface for SCPM configuration facilities
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 * Copyright 2002-2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef SCPM_CONF_H
#define SCPM_CONF_H

#include <string>

class SCPM_conf
{
public:

	friend class SCPM;

	struct not_available{};
	struct scpm_panic{};

	static SCPM_conf * GetHandle( ) { return handle; };
	std::string GetInitDir( ) { return initdir; };
	std::string GetLibDir( ) { return libdir; };
	std::string GetDBFile( ) { return dbfile; };
	std::string GetLogfile( ) { return logfile; };
	std::string GetRPMFile( ) { return rpmfile; };
	std::string GetResourceSet( ) { return resource_set; };
	std::string GetSetDir( ) { return libdir+"/resource_sets"; };
	std::string GetUserSetDir( ) { return "/var/lib/scpm/resource_sets"; };
	std::string GetGroupDir( ) { return libdir+"/resource_groups"; };
	std::string GetUserGroupDir( ) { return "/var/lib/scpm/resource_groups"; };
	std::string GetGroupsFile( ) { return groupsfile; };
	std::string GetProfileDir( ) { return profiledir; };
	std::string GetBootMode( ) { return boot_mode_str; };
	void	 SetOptions( int options );
	bool	 LogToStderr( ) { return log_to_stderr; };
	bool	 LogToFile( ) { return log_to_file; };
	bool	 LogToSyslog( ) { return log_to_syslog; };
	bool	 LogWarninig( ) { return log_warning; };
	bool	 LogError( ) { return log_warning; };
	bool	 Debug( ) { return log_debug; };
	bool	 Force( ) { return force_mode; };
	bool	 Skip( ) { return skip_mode; };
	bool	 Quiet( ) { return quiet; };
	bool	 Verbose( ) { return verbose; };
	bool	 ProgressBar( ) { return progress_bar; };
	bool	 BootMode( ) { return boot_mode; };
	bool	 ExitOnWarning( ) { return exit_warn; };
	bool	 SkipLoad( ) { return skip_load; };
	bool	 ForceDB( ) { return force_db; };
	bool	 WriteBootConfig( ) { return write_boot_config; };
	bool	 ReadOnly( ) { return read_only; };
	unsigned int GetMaxBackups( ) { return max_backups; };
	bool	 WaitCommit( ) { return wait_commit; };

		// you can also set some values
	void SetLibDir( std::string value ) { libdir = value; changed = true; };
	void SetDBFile( std::string value ) { dbfile = value; changed = true; };
	void SetLogFile( std::string value ) { logfile = value; changed = true; };
	void SetResourceSet( std::string value ) { resource_set = value; changed = true; };
	void SetBootMode( bool value ) { boot_mode = value; };

		// support for setting some options
	struct io_error{};
	struct no_such_option{};
	void SaveOption( const std::string &option, const std::string &value )
		throw( io_error, no_such_option );

private:

	SCPM_conf();
	~SCPM_conf();
	void SaveConfig();

	std::string initdir;
	std::string dbfile;
	std::string libdir;
	std::string logfile;
	std::string rpmfile;
	std::string profiledir;
	std::string resource_set;
	std::string groupsfile;
	std::string boot_mode_str;
	bool log_to_syslog;
	bool log_to_stderr;
	bool log_to_file;
	bool log_debug;
	bool log_warning;
	bool log_error;
	bool force_mode;
	bool skip_mode;
	bool quiet;
	bool verbose;
	bool progress_bar;
	bool boot_mode;
	bool exit_warn;
	bool skip_load;
	bool force_db;
	bool write_boot_config;
	bool read_only;
	unsigned int max_backups;
	bool wait_commit;

	bool changed;
	std::string cfg_fname;

	static SCPM_conf *handle;
};

#endif

