/*
 * $Id: scpm_helpers.hh,v 1.3 2006/09/21 07:47:58 hschaa Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	SCPM main engine
 * File				 :	scpm_helpers.cc
 * Description :	defines helper functions for SCPM main module
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002-2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef SCPM_HELPERS_H
#define SCPM_HELPERS_H

#include <string>

#include <scdb.hh>
#include "resource_helpers.hh"
#include "profile.hh"
#include "journal.hh"

class SCPM_helpers
{
public:

	struct db_open_failed {};
	struct db_corrupt {};
	struct profile_not_found {};

	SCPM_helpers();
	~SCPM_helpers();

		// general SCDB helper functions
	int LoadSCDB( );
	void SaveSCDB( );
	void SaveMagic( );
	bool IsEnabled( );
	void SetEnabled( );
	void SetDisabled( );
	bool IsInitialized( );
	void SetInitialized( );
	bool SystemChanged( );
	void MakeLock( );
	void ReleaseLock( );
	bool Locked( );
	bool InstallationChanged( );
	void SaveInstallInfo( );

		// profile specific helpers
	bool ProfileExists( std::string profile );
	const std::string GetActiveProfile( );
	const std::vector<std::string> GetAllProfiles( );
	void SetActiveProfile( std::string profile );
	void CreateNewProfile( std::string profile_name );
	void CloneProfile( std::string profile, std::string source_profile );
	void RemoveProfile( std::string profile );
	void SetScript( std::string profile, std::string script_type, std::string script_name );
	std::string GetScript( std::string profile, std::string script_type );
	void SetDescription( std::string profile, std::string description );
	std::string GetDescription( std::string profile );
	void AddProfileResources( std::string profile );

		// resource specific helpers
	void CreateResources( );
	const std::vector<std::string> GetResourceTypes( );
	const std::vector<std::string> GetResourceNames( std::string &resource_type );
	void CopyResources( const std::string &src_profile, const std::string &dest_profile );

		// resource action specific helpers
	void DetermineResourceActions( Profile &active, Profile &dest, std::vector<Resource*> &to_start,
																 std::vector<Resource*> &to_stop, std::vector<Resource*> &keep_runnning );
	void StopResources( std::vector<Resource*> &to_stop );
	void StartResources( std::vector<Resource*> &to_start );
	void ResolveDeps( Profile &active_profile, Profile &dest_profile,
										std::vector<Resource*> &resources, std::vector<Resource*> &affected );
	void OrderResources( std::vector<Resource*> &resources, bool stop );

		// resource special save method helpers
	void ApplyAll( const std::string &resource_type, const std::string &resource_name );
	void PatchAll( const std::string &resource_type, const std::string &resource_name );

		// resource group stuff
	std::string &GetGroupDescription( const std::string &group );
	bool GroupActive( const std::string &group );
	bool GroupDeleted( const std::string &group );

		// other helpers
	void WriteBootInfo( );
	void ClearBootInfo( );
	void RunScript( const std::string &scriptfile );
	void RestoreResources( const std::string &profile, const std::vector<std::pair<std::string, std::string> > &resources );
	void SaveResources( const std::string &profile, const std::vector<std::pair<std::string, std::string> > &resources );
	void UpdateResources(	 );
	void CheckResources( std::vector<std::pair<std::string, std::string> > &resources );
	void AddBackup ( const std::vector<std::pair<std::string, std::string> > &resources );
	void RemoveBackup ( const std::vector<Resource*> &resources, const std::vector<std::string> &versions );
	void RestoreBackup ( const std::vector<Resource*> &resources, const std::vector<std::string> &versions );
	std::vector<Resource*> &GetResourcesWithBackup( std::string profile="", bool verbose=false );
	std::vector<Resource*> &GetResourcesByName( const std::string &profile,
                                              std::vector<std::pair<std::string,std::string> > resources );

		// journal stuff
	void PerformCommand( Journal::entry &e );
	void RollbackCommand( Journal::entry &e );
	void ProcessJournal( bool nowait=false );

		// resource group stuff
	void SetResourceGroup(const std::string &name, const std::vector<resource_entry_t> &group,
												const std::string &description );
	void DeleteResourceGroup( const std::string &group );
	void RenameResourceGroup( const std::string &group, const std::string &newgroup );
	void SetActiveResourceGroups( const std::vector<std::string> &groups );
	void GetActiveResourceGroups( std::vector<std::string> &groups );
	void ActivateResourceGroup( const std::string &group, bool auto_add=false );
	void DeactivateResourceGroup( const std::string &group, bool auto_remove=false );
	void ResetResourceGroup( const std::string &group );
	void ResetAllGroups( );
	void GetResourceGroup( const std::string &name, std::vector<resource_entry_t> &group,
												 std::string &description, bool nouser );
	bool stay_locked;

private:

	void SetScriptEnv( );
	void Call( std::string command );

	SCDB *db;
	Log *log;
	Resource_helpers *resource_h;

};

#endif
