/*
 * $Id: file.hh,v 1.2 2006/09/21 07:47:58 hschaa Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	file resource
 * File				 :	file.hh
 * Description :	handling module for file resource type
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002-2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef FILE_H
#define FILE_H

#include <resource.hh>
#include <log.hh>
#include <string>
#include <vector>
#include <iostream>

class File : public Resource
{
public:

	friend class Resource_helpers;

	std::string GetType( ) { return "file"; };

		// some compilers (at least gcc 3) may argue about overriding throw statements
		// so we add them here as dummies (will not be thrown here)
	bool Active( ) throw( Resource::not_available );
	bool Modified( ) throw( Resource::not_available );
	bool UpToDate( ) throw( Resource::not_available );
	bool NeedsUpdate( ) throw( Resource::not_available ) ;
	bool Save( ) throw( Resource::not_available );
	bool Restore( ) throw( Resource::not_available );
	bool Patch( const std::string &ref_profile ) throw( Resource::not_available );
	void UpdateData( ) throw( Resource::not_available );
	std::vector<std::string> GetDependencies( ) throw( Resource::not_available ) { std::vector<std::string> v; return v; };
	void ShowChanges( std::ostream &ouput, bool backup, std::string version="" ) throw( Resource::not_available );
	bool ModifiedInProfile( const std::string &destprofile ) throw( Resource::not_available );
	bool AddBackup( ) throw( Resource::not_available );
	bool HasSystemAlternative( ) throw( Resource::not_available );
	bool HasSystemBackup( ) throw( Resource::not_available );
	bool HasBackup( ) throw( Resource::not_available );
	bool RestoreBackup( const std::string * version = NULL ) throw( Resource::not_available );
	bool RemoveBackup( const std::string * version = NULL) throw( Resource::not_available );
	std::vector< std::pair<time_t, std::string> > GetBackupVersions( ) throw( Resource::not_available );
	bool Export( const std::string &root ) throw( Resource::not_available );
	bool Import( const std::string &root ) throw( Resource::not_available );

	bool Refresh( ) throw( Resource::not_available );
	void PushBackup( ) throw( Resource::not_available );
	void SetRoot( const std::string &path) throw( Resource::not_available );

private:

	File( std::string name_string, std::string type_string, bool allow_missing );
	~File( );

	Log *log;
	std::string postrestore;
	std::vector<std::string> files;
	static std::string type;
	bool is_directory;
	int backup_state;

	std::string ReadLink( const std::string &link );
	void SaveFile( const std::string &file, SCDBFile *f, std::string suffix="" ) throw( SCDBFile::write_error );
	bool RestoreFile( SCDBFile *f );
	void ResolveFiles( );
	std::string GetType( const std::string &file );
	bool FileModified( SCDBFile *fileh );
	void CreatePath( const std::string &path );
	
};


#endif
