/*
 * $Id: service.hh,v 1.3 2007/03/09 09:43:26 gleissner Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	service resource
 * File				 :	service.hh
 * Description :	handling module for service resource type
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2001 SuSE GmbH
 *					 2002-2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef SERVICE_H
#define SERVICE_H

#include <resource.hh>
#include <log.hh>
#include <string>
#include <vector>

class Service : public Resource
{
public:

	friend class Resource_helpers;

	std::string GetType( ) { return "service"; };

		// some compilers (at least gcc 3) may argue about overriding throw statements
		// so we add them here as dummies (will not be thrown here)
	bool Active( ) throw( Resource::not_available );
	bool Modified( ) throw( Resource::not_available );
	bool UpToDate( ) throw( Resource::not_available );
	bool Running( ) throw( Resource::not_available );
	bool Start( ) throw( Resource::not_available );
	bool Stop( ) throw( Resource::not_available );
	bool Restart( ) throw( Resource::not_available );
	bool Save( ) throw( Resource::not_available );
	bool Restore( ) throw( Resource::not_available );
	std::vector<std::string> GetDependencies( std::string resource_type ) throw( Resource::not_available );
	int	 GetStartPriority( ) throw( Resource::not_available );
	int	 GetStopPriority( ) throw( Resource::not_available );
	void UpdateData( ) throw( Resource::not_available );
	bool Refresh( ) throw( Resource::not_available );
	bool HasBackup( ) throw( Resource::not_available );
	bool AddBackup( ) throw( Resource::not_available );
	bool RestoreBackup( const std::string* version=NULL ) throw( Resource::not_available );
	bool RemoveBackup( const std::string * version = NULL) throw( Resource::not_available );
	std::vector< std::pair<time_t, std::string> > GetBackupVersions( ) throw( Resource::not_available );
	void ShowChanges( std::ostream &output, bool backup=false, std::string version="" ) throw( Resource::not_available );
	bool Export( const std::string &root ) throw( Resource::not_available );
	bool Import( const std::string &root ) throw( Resource::not_available );

private:

	Service( std::string name_string, std::string type_string, bool allow_missing );
	~Service( );

	static const std::string init_dir;
	std::vector<std::string> provides;
	std::string start_link;
	std::string stop_link;
	Log *log;

	struct link_not_found {};

	std::string ReadLink( std::string link );
	void RemoveLinks( );
	void CreateLink( std::string link );
	std::string GetStartLink( std::string directory );
	std::string GetStopLink( std::string directory );
	char GetRunlevel( );
	void DetermineInitDirs( std::vector<std::string> &rc_dirs );
	int CalcStartPriority( );
	int CalcStopPriority( );
};


#endif
