
/*
 * $Id: local_file.hh,v 1.2 2006/09/21 07:47:59 hschaa Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	SCPM database
 * File				 :	local_file.hh
 * Description :	Local file interface
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2002-2004 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef LOCAL_FILE_H
#define LOCAL_FILE_H
#include <iostream>
#include <string>
#include "scdb_file.hh"

class LocalFile : public SCDBFile
{
public:
	friend class SCDB;
	friend class SCDBUtil;

	void Save( std::istream &ins, bool auto_md5=false, std::string suffix="") throw( SCDBFile::write_error );
	void Restore( std::ostream &outs ) throw( SCDBFile::read_error, SCDBFile::write_error );
	void Remove( ) throw( SCDBFile::write_error );
	void Create( std::string suffix="" ) throw( SCDBFile::write_error );
	void SaveAttribs( struct stat &attribs ) throw( SCDBFile::write_error );
	struct stat &GetAttribs( ) throw( SCDBFile::read_error );
	std::istream *GetStream( ) throw( SCDBFile::read_error );
	std::string &RealMD5sum( );
	std::string GetBackupVersion();

private:
	LocalFile( SCDBNode<backend_node> &node, const std::string &resname, const std::string &restype,
						 const std::string &profile_name, bool &modified, bool is_backup=false );
	std::string &EncodeFilename( std::string &s );
	void GenerateWriteLocation(std::string suffix="" ) throw( SCDBFile::write_error );
	std::string GetSuffix( std::string &fname, std::string suffix="" );
	void WriteBackupInfo( std::string &fname ) throw( SCDBFile::write_error );
	void ReadBackupInfo( std::string &fname ) throw( SCDBFile::read_error );
	
};

#endif
