/*
 * $Id: scdb_node.hh,v 1.1.1.1 2005/05/11 09:03:12 gleissner Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	SCPM database
 * File				 :	scdb_node.hh
 * Description :	defines SCDB Node
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2002 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef SCDB_NODE_H
#define SCDB_NODE_H

#include <string>
#include <vector>

template<class Node> class SCDBNode
{
public:
	SCDBNode( std::string name ) { node_name = name; deleted=false; available=true; };
	virtual ~SCDBNode( ) {};

	struct no_such_node {};

	virtual void AddSub( Node &sub_node ) { sub_nodes.push_back( &sub_node ); };
	virtual void DelSub( unsigned int idx ) throw( no_such_node )
		{ 
			if ( idx>=sub_nodes.size() ) throw no_such_node();
			typename std::vector<Node*>::iterator i = sub_nodes.begin();
			while ( idx-- ) i++;
			sub_nodes.erase( i );
		};
	virtual unsigned int GetNumSub( ) { return sub_nodes.size(); };
	virtual Node &operator[]( unsigned int idx ) throw( no_such_node )
		{ 
			if ( idx>=sub_nodes.size() ) throw no_such_node();
			return *sub_nodes[idx];
		};

	std::string	GetName( ) { return node_name; };
	void		SetName( std::string name ) { node_name = name; };
	void		SetValue( std::string value ) { node_value = value; };
	std::string	GetValue( ) { return node_value; };
	void		SetDeleted( bool del ) { deleted = del; };
	bool		Deleted( ) { return deleted; };
	bool		Available( ) { return available; };

protected:

	std::string node_name;
	std::vector<Node*> sub_nodes;
	std::string node_value;
	bool deleted;
	bool available;

};

#endif
