/*
 * $Id: xml.hh,v 1.1.1.1 2005/05/11 09:03:12 gleissner Exp $
 *
 *
 * Project		 :	SCPM (System Configuration Profile Management)
 * Module			 :	SCPM database
 * File				 :	xml.hh
 * Description :	class for loading and saving SCDB from/to a local file in XML format
 * Author			 :	Joachim Gleissner <jg@suse.de>
 *
 * Copyright 2002 SuSE Linux AG
 *
 * Released under the terms of the GNU General Public License
 * (see file COPYRIGHT in project root directory).
 *
 */
#ifndef XML_H
#define XML_H

#include "scdb_node.hh"
#include <iostream>
#include <log.hh>

class XMLNode : public SCDBNode<XMLNode>
{
public:
	friend class XML;
	friend class SCDB;
	friend class SCDBFile;

		//void AddSub( XMLNode &sub_node ) { sub_nodes.push_back( &sub_node ); };
		//unsigned int GetNumSub( ) { return sub_nodes.size(); };
		//XMLNode &operator[]( unsigned int idx ) throw( no_such_node )
		//{
		//	if ( idx>=sub_nodes.size() ) throw no_such_node();
		//	return *sub_nodes[idx];
		//};

private:
	XMLNode( std::string name ) : SCDBNode<XMLNode>( name ) {};
	~XMLNode( ) {
		unsigned int num = sub_nodes.size();
		for ( unsigned int i=0; i<num; i++) delete sub_nodes[i];
	};

	std::streampos filepos;
};

class XML
{
public:
	XML( );
	~XML( );

	struct db_stats_t {
		int	 node_count;
		int	 deleted_node_count;
		long mem_usage;
	};

	struct op_failed{};

	XMLNode *GetRoot( ) { return root; };
	std::string GetSCDBVersion( ) { return scdb_version_str; };
	void	 SetSCDBVersion( std::string version ) { scdb_version_str = version; };
	std::string GetFormatVersion( ) { return format_version_str; };
	bool	 UpToDate( );

		// quick=true skips loading of resource and profile data
	bool Load( std::istream &input, bool quick=false ) throw( );
	void Save( std::ostream &output );
	void DumpTree( XMLNode &node, std::ostream &output );
	void LoadBranch( XMLNode &node ) throw( op_failed );
	XMLNode &AddNode( XMLNode &node, std::string node_name );
	void CreateSubTree( XMLNode &node, std::istream &input, bool quick ) throw( op_failed );
	

private:

	struct syntax_error
	{
	public:
		syntax_error( std::string k, std::string e, unsigned int l )
			{
				key=k; error=e; line=l;
			}
		std::string key;
		std::string error;
		unsigned int line;
	};

	struct end_of_tokens {};

	struct key_t {
		std::string str;
		bool is_begin_token;
		bool is_end_token;
		std::streampos fpos;
	} ;

	XMLNode *root;
	int node_count;
	long tree_size;
	std::string scdb_version_str;
	std::string format_version_str;
	db_stats_t db_stats;
	std::istream *dbistream;

	key_t GetKey( std::istream &input ) throw( syntax_error, end_of_tokens );

};

#endif
